#!/usr/bin/python3

import argparse
import json
import subprocess

_CLIENTS_LOCATION = "/data/clients/"
_KUBECTL_CLIENT_BINARIES_LOCATION = _CLIENTS_LOCATION + "kubectl/"
_HELM_CLIENT_BINARIES_LOCATION = _CLIENTS_LOCATION + "helm/"
_COMMON_BINARY_LOCATION = "/data/bin"
_KUBECTL_COMPATIBILITY_FILE = "/data/build/resources/kubectl_compatibility.json"
_HELM_COMPATIBILITY_FILE = "/data/build/resources/helm_compatibility.json"
_KUBECTL_COMPATIBILITY = {}
_HELM_COMPATIBILITY = {}


def setup_kubernetes_tools():
    major_version, minor_version = get_server_version()
    setup_kubectl(major_version, minor_version)
    setup_helm(major_version, minor_version)


def get_server_version():
    default_kubectl_binary = _KUBECTL_CLIENT_BINARIES_LOCATION + _KUBECTL_COMPATIBILITY["DEFAULT"] + "/kubectl"
    version_json_string = execute_command(default_kubectl_binary + " version -o json")
    print("Kubectl Version : " + version_json_string)
    version_json = json.loads(version_json_string)
    major_version = version_json["serverVersion"]["major"]
    minor_version = version_json["serverVersion"]["minor"]
    print("Server version is " + str(major_version) + "." + str(minor_version))
    return major_version, minor_version


def setup_helm(major_version, minor_version):
    print("Setting up Helm for " + major_version + "." + minor_version)
    helm_client_version = get_helm_binary(major_version, minor_version)
    print("Setting up Helm version " + helm_client_version)
    execute_command("cp " + _HELM_CLIENT_BINARIES_LOCATION + helm_client_version + "/helm " + _COMMON_BINARY_LOCATION)


def setup_kubectl(major_version, minor_version):
    print("Setting up Kubectl for " + major_version + "." + minor_version)
    kubectl_client_version = get_kubectl_binary(major_version, minor_version)
    print("Setting up Kubectl version " + kubectl_client_version)
    execute_command(
        "cp " + _KUBECTL_CLIENT_BINARIES_LOCATION + kubectl_client_version + "/kubectl " + _COMMON_BINARY_LOCATION)


def get_kubectl_binary(major_version, minor_version):
    for binary, versions in _KUBECTL_COMPATIBILITY.items():
        if binary == "DEFAULT":
            continue
        for version in versions:
            if str(version).strip() == get_major_minor_version_string(major_version, minor_version):
                return binary
    return _KUBECTL_COMPATIBILITY["DEFAULT"]


def get_helm_binary(major_version, minor_version):
    for binary, versions in _HELM_COMPATIBILITY.items():
        if binary == "DEFAULT":
            continue
        for version in versions:
            if str(version).strip() == get_major_minor_version_string(major_version, minor_version):
                return binary
    return _HELM_COMPATIBILITY["DEFAULT"]


def get_major_minor_version_string(major_version, minor_version):
    return str(major_version).strip() + "." + str(minor_version).strip()


def load_compatibility():
    load_kubectl_compatibility()
    load_helm_compatibility()


def load_kubectl_compatibility():
    with open(_KUBECTL_COMPATIBILITY_FILE) as kubectl_config:
        kubectl_compatibility_json = json.load(kubectl_config)
    for binary in kubectl_compatibility_json:
        _KUBECTL_COMPATIBILITY[binary] = kubectl_compatibility_json[binary]


def load_helm_compatibility():
    with open(_HELM_COMPATIBILITY_FILE) as helm_config:
        helm_compatibility_json = json.load(helm_config)
    for binary in helm_compatibility_json:
        _HELM_COMPATIBILITY[binary] = helm_compatibility_json[binary]


def execute_command(cmd, timeout=300):
    """
    Executes command as shell command.
    :param cmd:  command to run
    :param timeout:  timeout for command. Default is 5 min
    :return: Output of command
    :raises:
    TimeoutExpired : if command takes more than given timeout
    CalledProcessError:  if failed to execute command.
    """
    try:
        output = subprocess.check_output(cmd, stdin=None, bufsize=-1, shell=True, timeout=timeout)
        return output.rstrip().decode()
    except subprocess.CalledProcessError as exc:
        msg = "Command %s returned non-zero code %s: %s" % (cmd, exc.returncode, exc.output)
        raise Exception(msg)
    except subprocess.SubprocessError as exc:
        msg = "Failed to execute command %s with error %s" % (cmd, str(exc))
        raise Exception(msg)
    except OSError as exc:
        msg = "Failed to execute command %s with error %s" % (cmd, str(exc))
        raise Exception(msg)


def parse_args():
    parser = argparse.ArgumentParser(description="Configure Kubernetes Tools")
    args, _ = parser.parse_known_args()
    return args


def main(options):
    print("Options by user are " + str(options))
    load_compatibility()
    setup_kubernetes_tools()


if __name__ == "__main__":
    ARGS = parse_args()
    main(ARGS)
